local function rotate_right(t, r)
	local new_t= {}
	for n= 1, #t do
		local index= ((n - r - 1) % #t) + 1
		new_t[n]= t[index]
	end
	return new_t
end

local function rotate_left(t, r)
	local new_t= {}
	for n= 1, #t do
		local index= ((n + r - 1) % #t) + 1
		new_t[n]= t[index]
	end
	return new_t
end

local function scan_tearing()
	local pos_spline= {
		"spline", loop= true, {"*", "y_offset", 1/16},
			0, 0,
	}
	local point_set= {32, 192, 32}
	for i= 16, 512, 16 do
		point_set[#point_set+1]= 0
	end
	for p= 1, #point_set do
		local sub_spline= {"spline", loop= true, {"*", "music_beat", 8}}
		for i= 1, #point_set do
			sub_spline[#sub_spline+1]= point_set[i]
		end
		pos_spline[#pos_spline+1]= sub_spline
		point_set= rotate_right(point_set, 1)
	end
	return {start_beat= 0, end_beat= 300, pos_spline}
end
local scan_tear_mod= scan_tearing()
rec_print_table(scan_tear_mod)
local function receptor_only_phase(amp)
	return {"phase", "dist_beat", {default= {0, 0, 0, 0}, {-.01, .01, 0, amp}}}
end
local function note_only_phase(amp)
	return {"phase", "dist_beat", {default= {0, 0, 0, 0}, {.01, 10, 0, amp}}}
end
local start_dist_beat= {"-", "music_beat", "start_beat"}
local end_dist_beat= {"-", "end_beat", "music_beat"}
local function receptor_vibrate(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"square", {"*", "music_beat", math.pi*16}},
		 receptor_only_phase(amp),
	}}
end
local function receptor_move(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		receptor_only_phase(amp)
	}
end
local function receptor_thump(start_beat, end_beat, amp)
	local durp= end_beat - start_beat
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"sin", {"*", start_dist_beat, math.pi / durp}},
		 receptor_only_phase(amp),
	}}
end
local function receptor_circle_x(start_beat, end_beat, amp)
	local durp= end_beat - start_beat
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"cos", {"*", start_dist_beat, math.pi * 2 / durp}},
		 receptor_only_phase(amp),
	}}
end
local function receptor_circle_y(start_beat, end_beat, amp)
	local durp= end_beat - start_beat
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"sin", {"*", start_dist_beat, math.pi * 2 / durp}},
		 receptor_only_phase(amp),
	}}
end
local function note_hifi_wave(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"sin", {"*", start_dist_beat, math.pi * 16}},
		 amp,
	}}
end
local function move(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		amp}
end
local function vibrate(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"square", {"*", "music_beat", math.pi * 16}},
		 amp,
	}}
end
local function tween(start_beat, end_beat, amp)
	local durp= end_beat - start_beat
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*", start_dist_beat, amp/durp}}
end
local function rev_tween(start_beat, end_beat, amp)
	local durp= end_beat - start_beat
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*", end_dist_beat, amp/durp}}
end
local function note_slow_vibe(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"square", {"*", "music_beat", math.pi/4}},
		 note_only_phase(amp),
	}}
end
local function note_vibe(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		{"*",
		 {"square", {"*", "music_beat", math.pi * 16}},
		 note_only_phase(amp),
	}}
end
local function note_move(start_beat, end_beat, amp)
	return {
		start_beat= start_beat, end_beat= end_beat,
		note_only_phase(amp)
	}
end
local mods= {
	field= {
		trans_rot_x= {
			move(76, 76.5, math.pi / -12),
			move(76.5, 77, 2 * math.pi / -12),
			move(77, 77.5, 3 * math.pi / -12),
			move(77.5, 78, 4 * math.pi / -12),
			move(78.25, 78.5, 4 * math.pi / -12),
			move(78.75, 79, 4 * math.pi / -12),
			move(79, 80, 2 * math.pi / -12),
			vibrate(79, 80, 2 * math.pi / -12),
			move(128, 130, math.pi / 4),
			move(130, 132, math.pi / -4),
			move(132, 133, 0),
			move(133, 134, math.pi / 4),
			move(134, 135, 2*math.pi / 4),
			move(135, 136, math.pi),
		},
	},
	columns= {
		{
			column_pos_x= {
				move(40, 44, -32),
				move(44, 46, -64),
				move(46, 48, -128),
				move(104, 108, -64),
				move(108, 109, -128),
				move(109, 111, -192),
				move(111, 111.5, -256),
				move(111.5, 112, -320),
				tween(136, 144, 64),
				move(144, 240, 64),
				rev_tween(240, 256, 64),
				move(192, 196, -32),
				move(196, 200, -64),
				move(200, 202, -96),
				move(202, 204, -128),
				move(204, 205, -96),
				move(205, 206, -64),
				move(206, 207, -32),
				vibrate(207, 208, 128),
				move(224, 228, -32),
				move(228, 232, -64),
				move(232, 234, -96),
				move(234, 236, -128),
				move(236, 237, -64),
				move(237, 238, -0),
				move(238, 239, 64),
				vibrate(239, 240, 128),
			},
			note_alpha= {
				move(48, 48.5, -1),
				note_hifi_wave(48, 48.5, 1),
				move(55, 55.5, -1),
				note_hifi_wave(55, 55.5, 1),
				move(58, 59, -1),
				note_hifi_wave(58, 59, 1),
				move(66, 67, -1),
				move(68, 69, -1),
				note_hifi_wave(68, 69, 1),
				move(73, 74, -1),
				note_hifi_wave(73, 74, 1),
				move(81, 82, -1),
				note_hifi_wave(81, 82, 1),
				move(82, 82.5, -1),
				move(86, 87, -1),
				note_hifi_wave(86, 87, 1),
				move(93, 94, -1),
				note_hifi_wave(93, 94, 1),
				move(104, 112, -1),
				note_hifi_wave(104, 112, 1),
				move(144, 146, -1),
				note_hifi_wave(144, 146, 1),
				move(148.5, 149.5, -1),
				note_hifi_wave(148.5, 149.5, 1),
				move(150, 155, -1),
				note_hifi_wave(150, 154, 1),
				move(161.5, 163, -1),
				note_hifi_wave(161.5, 162, 1),
				move(172, 174, -1),
				note_hifi_wave(172, 174, 1),
				move(176, 179, -1),
				note_hifi_wave(176, 178, 1),
				move(182, 186, -1),
				note_hifi_wave(182, 186, 1),
				move(197, 199, -1),
				note_hifi_wave(197, 199, 1),
			},
			note_glow= {
				vibrate(144, 240, -1),
			},
			note_pos_x= {
				scan_tear_mod,
				receptor_vibrate(16.75, 17.25, 32),
				receptor_circle_x(29.5, 31.5, 64),
				receptor_vibrate(36, 39, 64),
				receptor_vibrate(80, 81, 32),
				receptor_vibrate(88, 89, 32),
				note_move(144, 240, -32),
				note_vibe(144, 240, 32),
				receptor_circle_x(160.75, 162.75, 128),
				receptor_circle_x(167.5, 169.5, 128),
				receptor_circle_x(177, 177.5, 64),
				receptor_move(209, 210, -64),
				receptor_circle_x(220, 222, 64),
				receptor_circle_x(238, 240, 64),
			},
			note_pos_y= {
				receptor_vibrate(24, 25, 32),
				receptor_circle_y(29.5, 31.5, 64),
				receptor_vibrate(32, 35, 64),
				receptor_vibrate(40, 43, 64),
				vibrate(96, 98, 64),
				receptor_circle_y(160.75, 162.75, 128),
				receptor_circle_y(167.5, 169.5, 128),
				receptor_move(172, 174, 64),
				receptor_move(173, 174, 64),
				receptor_move(174.75, 175, 64),
				receptor_vibrate(175, 176, 128),
				receptor_vibrate(185, 186, 128),
				receptor_vibrate(194, 196, 128),
				receptor_vibrate(202, 204, 128),
				receptor_move(204, 206, 64),
				receptor_move(206, 207, 128),
				receptor_vibrate(217, 218, 128),
				receptor_vibrate(224, 225, 128),
				receptor_circle_y(220, 222, 64),
				receptor_vibrate(230, 232, 128),
				receptor_vibrate(234, 236, 128),
				receptor_circle_y(238, 240, 64),
			},
			note_rot_z= {
				tween(136, 144, -math.pi/2),
				move(144, 240, -math.pi/2),
				note_move(144, 240, math.pi/4),
				note_vibe(144, 240, -math.pi/4),
				rev_tween(240, 256, -math.pi/2),
			},
		},
		{
			column_pos_x= {
				move(40, 42, -16),
				move(42, 44, 16),
				move(44, 45, 32),
				move(45, 46, -32),
				move(46, 47, -64),
				move(47, 48, 64),
				move(104, 106, -32),
				move(106, 108, 32),
				move(108, 109, -32),
				move(109, 110, -96),
				move(110, 111, 32),
				move(111, 111.5, -32),
				move(111.5, 112, -96),
				tween(136, 144, -64),
				move(144, 240, -64),
				rev_tween(240, 256, -64),
				move(192, 196, -16),
				move(196, 200, -32),
				move(200, 202, -48),
				move(202, 204, -64),
				move(204, 205, -48),
				move(205, 206, -32),
				move(206, 207, -16),
				vibrate(207, 208, 128),
				move(224, 228, -16),
				move(228, 232, -32),
				move(232, 234, -48),
				move(234, 236, -64),
				move(236, 237, -32),
				move(237, 238, -0),
				move(238, 239, 32),
				vibrate(239, 240, 128),
			},
			note_alpha= {
				move(48.25, 48.75, -1),
				note_hifi_wave(48.25, 48.75, 1),
				move(52.75, 53.25, -1),
				note_hifi_wave(52.75, 53.25, 1),
				move(56, 57, -1),
				note_hifi_wave(56, 57, 1),
				move(60, 60.5, -1),
				note_hifi_wave(60, 60.5, 1),
				move(61, 61.5, -1),
				note_hifi_wave(61, 61.5, 1),
				move(63, 63.5, -1),
				note_hifi_wave(63, 63.5, 1),
				move(65, 66, -1),
				note_hifi_wave(65, 66, 1),
				move(69, 71, -1),
				note_hifi_wave(69, 71, 1),
				move(84, 85, -1),
				note_hifi_wave(84, 85, 1),
				move(86, 87, -1),
				note_hifi_wave(86, 87, 1),
				move(92, 93, -1),
				note_hifi_wave(92, 93, 1),
				move(94, 95, -1),
				note_hifi_wave(96, 98, 1),
				move(96, 99, -1),
				move(102, 104, -1),
				note_hifi_wave(102, 104, 1),
				move(106, 112, -1),
				note_hifi_wave(106, 112, 1),
				move(148, 150, -1),
				note_hifi_wave(148, 150, 1),
				move(151, 154, -1),
				note_hifi_wave(151, 154, 1),
				move(154.5, 158, -1),
				note_hifi_wave(154.5, 158, 1),
				move(161, 163, -1),
				note_hifi_wave(161, 162, 1),
				move(164, 167, -1),
				note_hifi_wave(164, 167, 1),
				move(168.5, 171, -1),
				note_hifi_wave(168.5, 171, 1),
				move(172.5, 174, -1),
				note_hifi_wave(172.5, 174, 1),
				move(177, 179, -1),
				note_hifi_wave(177, 178, 1),
				move(181, 186, -1),
				note_hifi_wave(181, 186, 1),
				move(188, 191, -1),
				note_hifi_wave(188, 191, 1),
				move(196, 197, -1),
				note_hifi_wave(196, 197, 1),
				move(201, 203, -1),
				note_hifi_wave(200, 202, 1),
			},
			note_glow= {
				vibrate(144, 240, -1),
			},
			note_pos_x= {
				scan_tear_mod,
				receptor_vibrate(16.5, 16.75, 32),
				receptor_vibrate(17.25, 17.5, 32),
				receptor_circle_x(29, 31, 64),
				receptor_vibrate(80.25, 81.25, 32),
				receptor_vibrate(81.5, 82.5, 32),
				note_move(144, 240, 32),
				note_vibe(144, 240, -32),
				receptor_circle_x(160.5, 162.5, 128),
				receptor_circle_x(167, 169, 128),
				receptor_circle_x(176.5, 177, 64),
				receptor_circle_x(177.25, 176.75, 64),
				receptor_move(208.5, 209.5, 64),
				receptor_move(209.25, 210.25, -64),
				receptor_circle_x(220.5, 222.5, 64),
				receptor_circle_x(237, 239, 64),
			},
			note_pos_y= {
				receptor_thump(20, 21, 64),
				receptor_circle_y(29, 31, 64),
				receptor_vibrate(48, 50, 128),
				vibrate(96, 98, 64),
				receptor_circle_y(160.5, 162.5, 128),
				receptor_circle_y(167, 169, 128),
				receptor_move(172.5, 174, 64),
				receptor_move(173.5, 174, 64),
				receptor_move(174.5, 175, 64),
				receptor_vibrate(175, 176, 128),
				receptor_move(204.5, 206.25, 64),
				receptor_move(206.25, 207, 128),
				receptor_vibrate(225, 226, 128),
				receptor_circle_y(220.5, 222.5, 64),
				receptor_vibrate(228, 230, 128),
				receptor_vibrate(232, 234, 128),
				receptor_circle_y(237, 239, 64),
			},
			note_rot_z= {
				tween(136, 144, math.pi/2),
				move(144, 240, math.pi/2),
				note_move(144, 240, -math.pi/4),
				note_vibe(144, 240, math.pi/4),
				rev_tween(240, 256, math.pi/2),
			},
		},
		{
			column_pos_x= {
				move(40, 42, 16),
				move(42, 44, -16),
				move(44, 45, -32),
				move(45, 46, 32),
				move(46, 47, 64),
				move(47, 48, -64),
				move(104, 106, 32),
				move(106, 108, -32),
				move(108, 109, 32),
				move(109, 110, 96),
				move(110, 111, -32),
				move(111, 111.5, 32),
				move(111.5, 112, 96),
				tween(136, 144, 64),
				move(144, 240, 64),
				rev_tween(240, 256, 64),
				move(192, 196, 16),
				move(196, 200, 32),
				move(200, 202, 48),
				move(202, 204, 64),
				move(204, 205, 48),
				move(205, 206, 32),
				move(206, 207, 16),
				vibrate(207, 208, 128),
				move(224, 228, 16),
				move(228, 232, 32),
				move(232, 234, 48),
				move(234, 236, 64),
				move(236, 237, 32),
				move(237, 238, 0),
				move(238, 239, -32),
				vibrate(239, 240, 128),
			},
			note_alpha= {
				move(48.5, 49, -1),
				note_hifi_wave(48.5, 49, 1),
				move(49, 50, -1),
				note_hifi_wave(49, 50, 1),
				move(52.25, 52.75, -1),
				note_hifi_wave(52.25, 52.75, 1),
				move(53, 54.5, -1),
				note_hifi_wave(53, 54.5, 1),
				move(60.5, 61, -1),
				note_hifi_wave(60.5, 61, 1),
				move(61.5, 62.5, -1),
				move(64.5, 65, -1),
				note_hifi_wave(64.5, 65, 1),
				move(72, 73, -1),
				note_hifi_wave(72, 73, 1),
				move(75, 78, -1),
				note_hifi_wave(75, 78, 1),
				move(80, 82, -1),
				note_hifi_wave(80, 82, 1),
				move(84.5, 85.5, -1),
				note_hifi_wave(81, 82, 1),
				move(88, 89, -1),
				note_hifi_wave(88, 89, 1),
				move(92.5, 93.5, -1),
				note_hifi_wave(89, 91, 1),
				move(100, 102, -1),
				note_hifi_wave(100, 102, 1),
				move(106, 112, -1),
				note_hifi_wave(106, 112, 1),
				move(144, 145, -1),
				note_hifi_wave(144, 145, 1),
				move(146, 147, -1),
				move(149, 152, -1),
				note_hifi_wave(149, 152, 1),
				move(155, 159, -1),
				note_hifi_wave(155, 159, 1),
				move(162, 163, -1),
				move(165, 167, -1),
				note_hifi_wave(165, 167, 1),
				move(169, 171, -1),
				note_hifi_wave(169, 171, 1),
				move(174, 175, -1),
				note_hifi_wave(174, 175, 1),
				move(178, 179, -1),
				move(183, 186, -1),
				note_hifi_wave(183, 186, 1),
				move(188, 191, -1),
				note_hifi_wave(188, 191, 1),
				move(196, 197, -1),
				note_hifi_wave(196, 197, 1),
				move(200, 203, -1),
				note_hifi_wave(200, 202, 1),
			},
			note_glow= {
				vibrate(144, 240, -1),
			},
			note_pos_x= {
				scan_tear_mod,
				receptor_vibrate(16.25, 16.5, 32),
				receptor_vibrate(17.5, 17.75, 32),
				receptor_circle_x(28.5, 30.5, 64),
				receptor_vibrate(80.5, 81.5, 32),
				receptor_vibrate(81.25, 82.25, 32),
				note_move(144, 240, -32),
				note_vibe(144, 240, 32),
				receptor_circle_x(160.25, 162.25, 128),
				receptor_circle_x(167, 169, 128),
				receptor_circle_x(176.25, 176.75, 64),
				receptor_circle_x(177.5, 178, 64),
				receptor_move(208.25, 209.25, 64),
				receptor_move(209.5, 210.5, -64),
				receptor_circle_x(221, 223, 64),
				receptor_circle_x(236, 238, 64),
				receptor_circle_x(237.5, 239.5, 64),
			},
			note_pos_y= {
				receptor_circle_y(28.5, 30.5, 64),
				receptor_vibrate(48, 50, 128),
				receptor_circle_y(160.25, 162.25, 128),
				receptor_circle_y(167, 169, 128),
				receptor_move(174.25, 175, 64),
				receptor_vibrate(175, 176, 128),
				receptor_move(205, 206.5, 64),
				receptor_move(206.5, 207, 128),
				receptor_vibrate(225, 226, 128),
				receptor_circle_y(221, 223, 64),
				receptor_vibrate(228, 230, 128),
				receptor_vibrate(232, 234, 128),
				receptor_circle_y(236, 238, 64),
				receptor_circle_y(237.5, 239.5, 64),
			},
			note_rot_z= {
				tween(136, 144, math.pi/2),
				move(144, 240, math.pi/2),
				note_move(144, 240, -math.pi/4),
				note_vibe(144, 240, math.pi/4),
				rev_tween(240, 256, math.pi/2),
			},
		},
		{
			column_pos_x= {
				move(40, 44, 32),
				move(44, 46, 64),
				move(46, 48, 128),
				move(104, 108, 64),
				move(108, 109, 128),
				move(109, 111, 192),
				move(111, 111.5, 256),
				move(111.5, 112, 320),
				tween(136, 144, -64),
				move(144, 240, -64),
				rev_tween(240, 256, -64),
				move(192, 196, 32),
				move(196, 200, 64),
				move(200, 202, 96),
				move(202, 204, 128),
				move(204, 205, 96),
				move(205, 206, 64),
				move(206, 207, 32),
				vibrate(207, 208, 128),
				move(224, 228, 32),
				move(228, 232, 64),
				move(232, 234, 96),
				move(234, 236, 128),
				move(236, 237, 64),
				move(237, 238, 0),
				move(238, 239, -64),
				vibrate(239, 240, 128),
			},
			note_alpha= {
				move(48.75, 49.25, -1),
				note_hifi_wave(48.75, 49.25, 1),
				move(52, 52.5, -1),
				note_hifi_wave(52, 52.5, 1),
				move(57, 58, -1),
				note_hifi_wave(57, 58, 1),
				move(64, 65, -1),
				move(71, 72, -1),
				note_hifi_wave(71, 72, 1),
				move(74, 75, -1),
				move(85, 86, -1),
				note_hifi_wave(85, 86, 1),
				move(89, 91, -1),
				note_hifi_wave(89, 91, 1),
				move(94, 95, -1),
				move(100, 102, -1),
				note_hifi_wave(100, 102, 1),
				move(104, 112, -1),
				note_hifi_wave(104, 112, 1),
				move(156, 158, -1),
				note_hifi_wave(156, 158, 1),
				move(164.5, 167, -1),
				note_hifi_wave(164.5, 167, 1),
				move(168, 171, -1),
				note_hifi_wave(168, 171, 1),
				move(176, 179, -1),
				note_hifi_wave(176, 178, 1),
				move(180, 181, -1),
				note_hifi_wave(180, 181, 1),
				move(197, 199, -1),
				note_hifi_wave(197, 199, 1),
				move(200, 203, -1),
				note_hifi_wave(200, 202, 1),
			},
			note_glow= {
				vibrate(144, 240, -1),
			},
			note_pos_x= {
				scan_tear_mod,
				receptor_vibrate(16, 16.25, 32),
				receptor_vibrate(17.75, 18, 32),
				receptor_circle_x(28, 30, 64),
				receptor_vibrate(36, 39, 64),
				receptor_vibrate(81, 82, 32),
				receptor_vibrate(89, 90, 32),
				note_move(144, 240, 32),
				note_vibe(144, 240, -32),
				receptor_circle_x(160, 162, 128),
				receptor_circle_x(167.5, 169.5, 128),
				receptor_circle_x(176, 176.5, 64),
				receptor_move(208, 209, 64),
				receptor_circle_x(221.5, 223.5, 64),
				receptor_circle_x(236.5, 238.5, 64),
			},
			note_pos_y= {
				receptor_vibrate(24, 25, 32),
				receptor_circle_y(28, 30, 64),
				receptor_vibrate(32, 35, 64),
				receptor_vibrate(40, 43, 64),
				receptor_circle_y(160, 162, 128),
				receptor_circle_y(167.5, 169.5, 128),
				receptor_move(174, 175, 64),
				receptor_vibrate(175, 176, 128),
				receptor_vibrate(184, 185, 128),
				receptor_vibrate(192, 194, 128),
				receptor_vibrate(200, 202, 128),
				receptor_move(205.5, 206.75, 64),
				receptor_move(206.75, 207, 128),
				receptor_vibrate(216, 217, 128),
				receptor_vibrate(224, 225, 128),
				receptor_circle_y(221.5, 223.5, 64),
				receptor_vibrate(230, 232, 128),
				receptor_vibrate(234, 236, 128),
				receptor_circle_y(236.5, 238.5, 64),
			},
			note_rot_z= {
				tween(136, 144, -math.pi/2),
				move(144, 240, -math.pi/2),
				note_move(144, 240, math.pi/4),
				note_vibe(144, 240, -math.pi/4),
				rev_tween(240, 256, -math.pi/2),
			},
		},
	},
}
for i= 1, #mods.columns do
	mods.columns[i].receptor_alpha= mods.columns[i].note_alpha
end

local frame= Def.Actor{
	OnCommand= function(self)
		local screen= SCREENMAN:GetTopScreen()
		for i, pn in ipairs(GAMESTATE:GetEnabledPlayers()) do
			local field= find_notefield_in_gameplay(screen, pn)
			if field then
				for mod_name, mod_set in pairs(mods.field) do
					field["get_"..mod_name](field):add_managed_mod_set(mod_set)
				end
				for i, col in ipairs(field:get_columns()) do
					for mod_name, mod_set in pairs(mods.columns[i]) do
						col["get_"..mod_name](col):add_managed_mod_set(mod_set)
					end
				end
			end
		end
	end
}

return frame
